/************************************************************************
     File:        MyArcBall.h

				- note This code is deprecated as of October 2007
				  the functionality has been put inside of "ArcBallCam.H"

				  this is kept around in case you want the more basic
				  implementation, but may not be kept up to date

     Author:     
                  Michael Gleicher, gleicher@cs.wisc.edu
     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
     
     Comment:    
									Written October, 2001, Michael L. Gleicher
									Updated October, 2005 for CS559 - Michael L. Gleicher

		 
									An arcball is a user interface for rotating an object around
									its very useful for looking around the world.

									The actual mathematics to implement it are elegant, but beyond the
									scope of CS559. This example code is provided for you to use
									without having to worry about the details of how it works.

		To use this code:
								1. Create an instance of the "ArcBall" in your window class
								2. When you draw, pull that matrix out of the ArcBall (using the
								    getMatrix method).
								3. When you get a mouse button push event, figure out where the event
								   happened (in normalized device coordinates) and call the "down"
								   method of the ArcBall
								4. When you get a drag event (and the button that is pushed down is
								   the one for moving the viewpoint)
    Note:
								1.	A quickly hacked together implementation of ArcBall, with lots of
									  bits lifted from Ken Shoemake's original source in Graphics Gems
									  made into a nice C++ thing

								2.	It is simplified a lot - the controller is always at the origin,
								3.	It has a fixed radius, it doesn't have constraints


   
     Platform:    Visio Studio.Net 2003

     CVS Info: Date checked in: $Date: 2007/10/22 15:38:30 $
               Revision number: $Revision: 1.3 $

*************************************************************************/

#ifndef MyArcBall_H
#define MyArcBall_H

// this typedef is useful for lots of stuff
typedef float HMatrix[4][4];

// if you need to pass that to OpenGL, try...
inline float* asGlMatrix(HMatrix m) { return (float*) m; };

//**************************************************************************
//
// We need a quick and dirty Quaternion class
// Note1 : If you're a 559 student, don't worry about how this works
// Note2 : If you know what a quaternion is and want to use it, you
//         probably don't want this one! it's a simple version only for the
//         simple arcball
//
//**************************************************************************
class Quat {
 public:
  Quat();						/* gives the identity */
  Quat(float x, float y, float z, float w);
  Quat(const Quat&);			/* copy constructor */

  // conversions
  void toMatrix(HMatrix) const;

  // operations
  Quat conjugate() const;
  // make multiply look like multiply
  Quat operator* (const Quat&) const;

  void renorm();

  // the data
  float x,y,z,w;
};


//**************************************************************************
//
// The arcball implementation
//
//**************************************************************************
class MyArcBall {
public:
	MyArcBall();

	// Simplified user interface
	void multMatrix();	// multiply matrix ontop of stack

	// Reset to a basic configuration
	// if you use the default (all zeros), you'll get the
	// identity transformation
	// otherwise, give a vector (x,y,z) that is the vector that
	// you want the world to rotate around. the magnitude of the vector
	// is the amount that you want to rotate (its going to be the magnitude
	// of the vector part of the quaternion)
	void reset(float x=0, float y=0, float z=0);		// reset back to identity transform

	// These assume that we're inside of 
	Quat start;			// orientation at the start of drag
	Quat now;			// quaternion "now" (while dragging)

	float downX, downY;	// where the mouse went down

	// call this when the mouse goes down
	void down(const float x, const float y);

	// this updates the cached positions - call it when the mouse is dragged
	void computeNow(const float nowX, const float nowY);

	// this gets the global matrix (start and now)
	void getMatrix(HMatrix) const;

	// Spin the ball by some vector - if you don't understand
	// how an arcball works, you probably don't care about this
	// but: basically you give it a vector to rotate the world around
	// - the length of the vector is how much to rotate around that
	//   axis
	void spin(float x, float y, float z);
};

#endif // MyArcBall_H
