// Minimal project 2 phase 1 
// Written by Mike Gleicher, while on a plane from MKE to EWR
// this was written as fast as possible, and designed to be 
// as simple as possible but still meet the minimal requirements

// do not look at this for an example of good code!
// its just to see how quickly i could through together an
// answer to the project

//     CVS Info: Date checked in: $Date: 2007/10/22 15:38:30 $
//               Revision number: $Revision: 1.3 $



#include <windows.h>
#include <Gl/gl.h>

#include <vector>
#include <utility>

// the FlTk headers have lots of warnings - these are bad, but there's not
// much we can do about them
#pragma warning(push)
#pragma warning(disable:4311)		// convert void* to long
#pragma warning(disable:4312)		// convert long to void*
#include <Fl/fl.h>
#include <Fl/Fl_Gl_Window.h>
#pragma warning(pop)

// i'll just keep everything as global variables, and use STL (not objects)
// again, this is for simplicity, not for "best practices"
std::vector< std::pair<float,float> > points;
size_t selected=0;
bool running = false;

// we need to make our own version of a window so we can redefine draw and handle
class Simple2D : public Fl_Gl_Window {
public:
	Simple2D() : Fl_Gl_Window(100,100,600,600,"Simple2D"), angle(0){}
  float angle;
private:
	// draw the squares - this assumes that the window is 500x500
	// and doesn't change (not a good thing to assume, but again, for
	// simplicity)
  void draw() {
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	// in practice, we should set up the coordinate system based on the actual
	// window size
	glOrtho(0,500,0,500,-1,1);
	// draw each point. if its selected, draw it in red
	for(size_t i=0; i<points.size(); i++) {
		if (selected == i)
				glColor3f(1.0f,0,0);
		else 	glColor3f(1.0,1.0,1.0);
		glRectf(points[i].first-20,points[i].second-20,points[i].first+20,points[i].second+20);
	}
	// if there is a selected point, make the little orbiting square
	if (selected < points.size()) {
		glColor3f(1.0f,1.0f,0.0f);
		glPushMatrix();
		glTranslatef(points[selected].first,points[selected].second,0);
		glRotatef(angle, 0, 0, 1);
		glRectf(25,-5,36,5);
		glPopMatrix();
	}
  };
  // completely keyboard driven UI
  // note: if we want to use the keyboard, we need to take focus when its given
  // to us
  int handle(int e) {
	  switch(e) {
		  case FL_ENTER:
		  case FL_FOCUS:
		  case FL_UNFOCUS:
			return 1;
		case FL_KEYDOWN: {
		 	int c = Fl::event_key();	  
			switch(c) {
				case 'n':	// select the next square
					selected = (selected + 1) % points.size();
					break;
				case 'a':	// add a new square (at the center)
					points.push_back(std::pair<float,float>(250,250));
					selected = points.size()-1;
					break;
				case FL_Right: // move selected to the right
					if (selected < points.size()) points[selected].first += 10;
					break;
				case FL_Left: // move selected to the left
					if (selected < points.size()) points[selected].first -= 10;
					break;
				case FL_Up: // move selected up
					if (selected < points.size()) points[selected].second += 10;
					break;
				case FL_Down: // move selected down
					if (selected < points.size()) points[selected].second -= 10;
					break;
				case 's':	// switch the running state 
					running = !running;
					break;
				default:	// any other key? pass along to the parent
					return Fl_Gl_Window::handle(e);
			};
						 };
			damage(1);
			return 1;
		default:	// we only handle keypresses - send everything else to the parent
			return Fl_Gl_Window::handle(e);
	  };
  };
};
// idle call back - assume the parameter is a Simple2D (window)
// advance the angle if we're running
void idleCB(void* w)
{
	if (running) {
		((Simple2D*)w)->angle += 5;
		((Simple2D*)w)->damage(1);
	}
}
// main - set up the window and let FlTk do its thing
int main(int, char**)
{
	points.push_back(std::pair<float,float>(100,100));
	points.push_back(std::pair<float,float>(400,400));
	Simple2D theWindow;
	theWindow.show();
	Fl::add_idle(idleCB,&theWindow);
	Fl::run();
}
